@@ -10,6 +10,7 @@ use Log::Log4perl;
 use Log::Log4perl::Level;
 use Log::Log4perl::Layout;
 use Log::Log4perl::Appender;
+use Log::Log4perl::Appender::String;
 use Log::Log4perl::Filter;
 use Carp;
 
@@ -25,8 +26,29 @@ our %APPENDER_BY_NAME = ();
 our $INITIALIZED = 0;
 our $NON_INIT_WARNED;
 
+    # Define the default appender that's used for formatting
+    # warn/die/croak etc. messages.
+our $STRING_APP_NAME = "_l4p_warn";
+our $STRING_APP      = Log::Log4perl::Appender->new(
+                          "Log::Log4perl::Appender::String",
+                          name => $STRING_APP_NAME);
+$STRING_APP->layout(Log::Log4perl::Layout::PatternLayout->new("%m"));
+our $STRING_APP_CODEREF = generate_coderef([[$STRING_APP_NAME, $STRING_APP]]);
+
 __PACKAGE__->reset();
 
+###########################################
+sub warning_render {
+###########################################
+    my($logger, @message) = @_;
+
+    $STRING_APP->string("");
+    $STRING_APP_CODEREF->($logger, 
+                          @message, 
+                          Log::Log4perl::Level::to_level($ALL));
+    return $STRING_APP->string();
+}
+
 ##################################################
 sub cleanup {
 ##################################################
@@ -335,14 +357,13 @@ sub generate_is_xxx_coderef {
 ##################################################
     my($return_token) = @_;
 
-    my $coderef    = '';
-    my $watch_code = '';
+    my $coderef    = sub { $return_token };
 
     if (defined $Log::Log4perl::Config::WATCHER) {
 
         my $cond = generate_watch_conditional();
 
-        $watch_code = <<EOL;
+        my $watch_code = <<EOL;
         my(\$logger, \$subname) = \@_;
         if($cond) {
             Log::Log4perl->init_and_watch();
@@ -350,13 +371,13 @@ sub generate_is_xxx_coderef {
             return \$logger->\$subname();
         }
 EOL
-    }
 
-    my $code = <<EOL;
-    \$coderef = sub { $watch_code return $return_token; };
+        my $code = <<EOL;
+        \$coderef = sub { $watch_code return $return_token; };
 EOL
 
-    eval $code or die "$@";
+        eval $code or die "$@";
+    }
 
     return $coderef;
 }
@@ -381,6 +402,12 @@ sub generate_watch_code {
             Log::Log4perl->init_and_watch();
                        
             my \$methodname = lc(\$level);
+
+                # Bump up the caller level by two, since
+                # we've artifically introduced additional levels.
+            local(\$Log::Log4perl::caller_depth);
+            \$Log::Log4perl::caller_depth += 2;
+
             \$logger->\$methodname(\@_); # send the message
                                          # to the new configuration
             return;        #and return, we're done with this incarnation
@@ -810,15 +837,14 @@ sub and_warn {
 #######################################################
   my $self = shift;
   my $msg = join("", @_[0 .. $#_]);
-  CORE::warn(callerline($msg));
+  CORE::warn(callerline($self->warning_render(@_[0 .. $#_])));
 }
 
 #######################################################
 sub and_die {
 #######################################################
   my $self = shift;
-  my $msg = join("", @_[0 .. $#_]);
-  die(callerline($msg));
+  die(callerline($self->warning_render(@_[0 .. $#_])));
 }
 
 ##################################################
@@ -871,14 +897,6 @@ sub logexit {
   exit $Log::Log4perl::LOGEXIT_CODE;
 }
 
-##################################################
-# for die and warn, carp long/shortmess return line #s and the like
-##################################################
-sub noop {
-##################################################
-  return @_;
-}
-
 ##################################################
 # clucks and carps are WARN level
 sub logcluck {
@@ -886,13 +904,14 @@ sub logcluck {
   my $self = shift;
 
   local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+  my $msg = $self->warning_render(@_);
 
   if ($self->is_warn()) {
-    my $message = Carp::longmess(@_);
+    my $message = Carp::longmess($msg);
     foreach (split(/\n/, $message)) {
       $self->warn("$_\n");
     }
-    Carp::cluck(noop($message));
+    Carp::cluck($msg);
   }
 }
 
@@ -902,13 +921,14 @@ sub logcarp {
   my $self = shift;
 
   local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+  my $msg = $self->warning_render(@_);
 
   if ($self->is_warn()) {
-    my $message = Carp::shortmess(@_);
+    my $message = Carp::shortmess($msg);
     foreach (split(/\n/, $message)) {
       $self->warn("$_\n");
     }
-    Carp::carp(noop($message)) if $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR;
+    Carp::carp($msg) if $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR;
   }
 }
 
@@ -920,8 +940,9 @@ sub logcroak {
   my $self = shift;
 
   local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+  my $msg = $self->warning_render(@_);
 
-  my $message = Carp::shortmess(@_);
+  my $message = Carp::shortmess($msg);
   if ($self->is_fatal()) {
     foreach (split(/\n/, $message)) {
       $self->fatal("$_\n");
@@ -929,7 +950,7 @@ sub logcroak {
   }
 
   $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
-      Carp::croak(noop($message)) : 
+      Carp::croak($msg) : 
         exit($Log::Log4perl::LOGEXIT_CODE);
 }
 
@@ -938,9 +959,10 @@ sub logconfess {
 ##################################################
   my $self = shift;
 
-#local $Carp::CarpLevel = 2;
+  my $msg = $self->warning_render(@_);
+  local $Carp::CarpLevel = 2;
 
-  my $message = Carp::longmess(@_);
+  my $message = Carp::longmess($msg);
   if ($self->is_fatal()) {
     foreach (split(/\n/, $message)) {
       $self->fatal("$_\n");
@@ -948,7 +970,7 @@ sub logconfess {
   }
 
   $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
-      die(noop($message)) :
+      confess($msg) :
         exit($Log::Log4perl::LOGEXIT_CODE);
 }
 
@@ -968,14 +990,17 @@ sub error_warn {
 sub error_die {
 ##################################################
   my $self = shift;
+
+  my $msg = $self->warning_render(@_);
+
   if ($self->is_error()) {
     $Log::Log4perl::caller_depth++;
-    $self->error(@_);
+    $self->error($msg);
     $Log::Log4perl::caller_depth--;
   }
 
   $Log::Log4perl::LOGDIE_MESSAGE_ON_STDERR ? 
-      $self->and_die(@_) :
+      $self->and_die($msg) :
         exit($Log::Log4perl::LOGEXIT_CODE);
 }
 
